import Acquisition
import AccessControl
import OFS
from OFS import content_types

from Globals import HTMLFile, MessageDialog, Persistent
from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String

from AccessControl.Permission import Permission

import string
import time
import sys
import types
import StringIO
import mimetypes
import re
import rfc822
import os
import urllib
import mimify


try:
    import DateTime
except: pass


import Imap
from ImapUtil import CreateStruct, makemap, Internaldate2tuple
import ImapFolder
import ImapMessage

import Sieve

from pprint import pformat

ConnectionError='IMAP Connection Error'

# Sieve Helper Class

class SieveScript:
	def __init__(self,name,active):
		self.name = name
		self.active = active



# Imap Root Folder (the zope Folder object)

class ImapMailbox(ImapFolder.ImapFolder,
    AccessControl.Role.RoleManager):
	'''
	ImapMailbox Product
	'''

	name='ImapMailbox'
	id='ImapMailbox'
	icon='misc_/ImapClient/Mailbox'
	meta_type='ImapMailbox'
	
	title='Mailbox'
	
	isAImapMailbox = 1
	
        _properties = (
                {'id':'title','type':'string','mode':'w'},
                {'id':'username','type':'string','mode':'w'},
		{'id':'password','type':'string','mode':'w'},
                {'id':'host','type':'string','mode':'w'},
                {'id':'Folder','type':'string','mode':'w'},		
                )


	manage_options=(
		{'label':'Contents','icon':icon,'action':'manage_main','target':'manage_main'},
		{'label':'ACL','icon':icon,'action':'manage_editACLForm','target':'manage_main'},
		{'label':'Sieve','icon':icon,'action':'manage_sieve','target':'manage_main'},
		{'label':'Properties','icon':icon,'action':'manage_propertiesForm','target':'manage_main'},
		{'label':'Security','icon':icon,'action':'manage_access','target':'manage_main'},
		)


	__ac_permissions__=(
		('View management screens',('manage_tabs','manage_main')),
		('Change permissions',('manage_access',)),
		('Change ImapMailbox',('manage_edit',),('Manager',)),
		)
		


	def __init__(self,id,title='Mailbox',username='',password='',host='localhost',userfolder='INBOX.'):

		self._v_conn = None
		self._folder_uptodate = 0
		self._v_last_selected = None


		self.id=id
		self.title=title
		self.username=username
		self.password=password
		self.host=host
		
		if string.upper(userfolder) != 'USER':
			self.isAMessageFolder = 1
		else:
			self.isAMessageFolder = 0
		
		ImapFolder.ImapFolder.__init__.im_func(self,id,userfolder)




	## Zope Management


	def manage_edit(self,title='Mailbox',username='',password='',host='localhost',userfolder='INBOX.',REQUEST=None):
		"""edit ImapMailbox"""
		self.title=title
		self.username=username
		self.password=password
		self.host=host
		self.Folder = userfolder

		if REQUEST is not None:
			return MessageDialog(
				title='Edited',
				message='<strong>%s</strong> has been edited.' % self.id,
				action = './manage_main'
				)

	## ObjectManager Patches


	def canBrowse(self):
		return self.isOpen()


	## Connection Handling

	def _connection(self):
       	    if not self.isOpen(): self._open()
            return self._v_conn


	def isOpen(self):
	    " quickly checks to see if the connection's open "
	    if not hasattr(self, '_v_conn'):
        	self._v_conn = None
	    if self._v_conn is None:
        	return 0
	    elif self._v_conn.noop()[0] != 'OK':
		 self._v_conn = None
		 return 0
	    else:
        	 return 1

	def _open(self):
	    """ open a connection """
	    try:
        	self._close()
	    except:
        	pass
	    self._v_conn = self.getNewConnection()


	def _close(self):
		M = self._v_conn
		M.close()
		M.logout()
		self._v_conn = None
		
		
	def getNewConnection(self):
	        try:
			conn=Imap.IMAP4(self.host)
			if conn.login(self.username, self.password):
                        	return conn
                        else:
                        	return None
	        except:
        		return None

	def getImapConnection(self, Folder=None, rw=0):
	       	if not Folder:
			Folder=self.Folder
		
        	M=self._v_conn=self._connection()
		
		if not M:
			raise "Connection Error"
		
		if Folder:
			resp = []
			if Folder[-1] == '.':
				Folder = Folder[:-1]

			if not hasattr(self, '_v_last_selected'):
        			self._v_last_selected = None

			if self._v_last_selected == (Folder,rw):
				return M
			else:
				self._v_last_selected = (Folder,rw)
				if rw: resp = M.select(Folder)
                		else:  resp = M.select(Folder,1)

                return M

	## Capabilities
	
	def getImapCapabilities(self):
		''' get IMAP Capabilities '''
                M=self.getImapConnection()
		return M.capabilities


	## SieveScript Support

	manage_sieve = HTMLFile('manage/sieve',globals())

	
	def _SieveConnection(self):
		try:
			con = Sieve.SIEVE(self.host)
			con.login(self.username,self.password)
			return con
		except:
			return None


	def getSieveScriptList(self):
		M = self._SieveConnection()
		
		if not M:
			raise "Sieve Connection Error"
			
		scriptlist = M.listscripts()[1]
		
		ret = []
		for name,active in scriptlist:
			ret.append(SieveScript(name,active))
		M.logout()
		return tuple(ret)

	manage_editSieveScriptForm = HTMLFile('manage/sievescript_edit',globals())

	def getSieveScript(self,name):
		''' edit a Sieve Script '''
		

		M = self._SieveConnection()
		
		if not M:
			raise "Sieve Connection Error"

		script = M.getscript(name)[1]
		M.logout()			
		return script
		
	def manage_editSieveScript(self,REQUEST):
		''' edit a Sieve Script '''
		
		name = REQUEST.get('scriptname','')
		script = REQUEST.get('scripttext','')

		M = self._SieveConnection()
		
		if not M:
			raise "Sieve Connection Error"
		if name and script:	

			M.putscript(name,script)
		else:
			return self.manage_sieve(self,REQUEST)

		M.logout()		
		if REQUEST is not None:
			return self.manage_sieve(self,REQUEST)

	def manage_activateSieveScript(self,REQUEST):
		''' activate a Sieve Script '''
		
		ids = REQUEST.get('ids',[])

		M = self._SieveConnection()
		
		if not M:
			raise "Sieve Connection Error"
		if ids:	

			M.setactive(ids[0])
		else:
			return self.manage_sieve(self,REQUEST)

		M.logout()		
		if REQUEST is not None:
			return self.manage_sieve(self,REQUEST)


	def manage_deleteSieveScript(self,REQUEST):
		''' delete Sieve Scripts '''
		
		ids = REQUEST.get('ids',[])

		M = self._SieveConnection()
		
		if not M:
			raise "Sieve Connection Error"
		if ids:	
			for id in ids:
				M.deletescript(id)
		else:
			return self.manage_sieve(self,REQUEST)

		M.logout()		
		if REQUEST is not None:
			return self.manage_sieve(self,REQUEST)
			


import Globals
for klass in (ImapMailbox,):
    Globals.default__class_init__(klass)







